@@ -33,18 +33,20 @@ class TeleBot:
         getMe
         sendMessage
         forwardMessage
+        deleteMessage
         sendPhoto
         sendAudio
         sendDocument
         sendSticker
         sendVideo
+        sendVideoNote
         sendLocation
         sendChatAction
         getUserProfilePhotos
         getUpdates
     """
 
-    def __init__(self, token, threaded=True, skip_pending=False):
+    def __init__(self, token, threaded=True, skip_pending=False, num_threads=2):
         """
         :param token: bot API token
         :return: Telebot object.
@@ -72,10 +74,12 @@ class TeleBot:
         self.inline_handlers = []
         self.chosen_inline_handlers = []
         self.callback_query_handlers = []
+        self.shipping_query_handlers = []
+        self.pre_checkout_query_handlers = []
 
         self.threaded = threaded
         if self.threaded:
-            self.worker_pool = util.ThreadPool()
+            self.worker_pool = util.ThreadPool(num_threads=num_threads)
 
     def set_webhook(self, url=None, certificate=None, max_connections=None, allowed_updates=None):
         return apihelper.set_webhook(self.token, url, certificate, max_connections, allowed_updates)
@@ -144,6 +148,9 @@ class TeleBot:
         new_inline_querys = []
         new_chosen_inline_results = []
         new_callback_querys = []
+        new_shipping_querys = []
+        new_pre_checkout_querys = []
+
         for update in updates:
             if update.update_id > self.last_update_id:
                 self.last_update_id = update.update_id
@@ -161,6 +168,11 @@ class TeleBot:
                 new_chosen_inline_results.append(update.chosen_inline_result)
             if update.callback_query:
                 new_callback_querys.append(update.callback_query)
+            if update.shipping_query:
+                new_shipping_querys.append(update.shipping_query)
+            if update.pre_checkout_query:
+                new_pre_checkout_querys.append(update.pre_checkout_query)
+
         logger.debug('Received {0} new updates'.format(len(updates)))
         if len(new_messages) > 0:
             self.process_new_messages(new_messages)
@@ -176,6 +188,10 @@ class TeleBot:
             self.process_new_chosen_inline_query(new_chosen_inline_results)
         if len(new_callback_querys) > 0:
             self.process_new_callback_query(new_callback_querys)
+        if len(new_pre_checkout_querys) > 0:
+            self.process_new_pre_checkout_query(new_pre_checkout_querys)
+        if len(new_shipping_querys) > 0:
+            self.process_new_shipping_query(new_shipping_querys)
 
     def process_new_messages(self, new_messages):
         self._append_pre_next_step_handler()
@@ -202,6 +218,12 @@ class TeleBot:
     def process_new_callback_query(self, new_callback_querys):
         self._notify_command_handlers(self.callback_query_handlers, new_callback_querys)
 
+    def process_new_shipping_query(self, new_shipping_querys):
+        self._notify_command_handlers(self.shipping_query_handlers, new_shipping_querys)
+
+    def process_new_pre_checkout_query(self, pre_checkout_querys):
+        self._notify_command_handlers(self.pre_checkout_query_handlers, pre_checkout_querys)
+
     def __notify_update(self, new_messages):
         for listener in self.update_listener:
             self._exec_task(listener, new_messages)
@@ -408,6 +430,15 @@ class TeleBot:
         return types.Message.de_json(
             apihelper.forward_message(self.token, chat_id, from_chat_id, message_id, disable_notification))
 
+    def delete_message(self, chat_id, message_id):
+        """
+        Use this method to delete message. Returns True on success. 
+        :param chat_id: in which chat to delete
+        :param message_id: which message to delete
+        :return: API reply.
+        """
+        return apihelper.delete_message(self.token, chat_id, message_id)
+
     def send_photo(self, chat_id, photo, caption=None, reply_to_message_id=None, reply_markup=None,
                    disable_notification=None):
         """
@@ -500,6 +531,22 @@ class TeleBot:
             apihelper.send_video(self.token, chat_id, data, duration, caption, reply_to_message_id, reply_markup,
                                  disable_notification, timeout))
 
+    def send_video_note(self, chat_id, data, duration=None, length=None, reply_to_message_id=None, reply_markup=None,
+                        disable_notification=None, timeout=None):
+        """
+        Use this method to send video files, Telegram clients support mp4 videos.
+        :param chat_id: Integer : Unique identifier for the message recipient — User or GroupChat id
+        :param data: InputFile or String : Video note to send. You can either pass a file_id as String to resend a video that is already on the Telegram server
+        :param duration: Integer : Duration of sent video in seconds
+        :param length: Integer : Video width and height, Can't be None and should be in range of (0, 640)
+        :param reply_to_message_id:
+        :param reply_markup:
+        :return:
+        """
+        return types.Message.de_json(
+            apihelper.send_video_note(self.token, chat_id, data, duration, length, reply_to_message_id, reply_markup,
+                                      disable_notification, timeout))
+
     def send_location(self, chat_id, latitude, longitude, reply_to_message_id=None, reply_markup=None,
                       disable_notification=None):
         """
@@ -549,7 +596,7 @@ class TeleBot:
         its typing status).
         :param chat_id:
         :param action:  One of the following strings: 'typing', 'upload_photo', 'record_video', 'upload_video',
-                        'record_audio', 'upload_audio', 'upload_document', 'find_location'.
+                        'record_audio', 'upload_audio', 'upload_document', 'find_location', 'record_video_note', 'upload_video_note'.
         :return: API reply. :type: boolean
         """
         return apihelper.send_chat_action(self.token, chat_id, action)
@@ -586,7 +633,8 @@ class TeleBot:
                                      reply_markup)
         return types.Message.de_json(result)
 
-    def set_game_score(self, user_id, score, force=None,chat_id=None, message_id=None, inline_message_id=None, edit_message=None):
+    def set_game_score(self, user_id, score, force=None, chat_id=None, message_id=None, inline_message_id=None,
+                       edit_message=None):
         result = apihelper.set_game_score(self.token, user_id, score, force, chat_id, message_id, inline_message_id,
                                           edit_message)
         if type(result) == bool:
@@ -600,6 +648,24 @@ class TeleBot:
             ret.append(types.GameHighScore.de_json(r))
         return ret
 
+    def send_invoice(self, chat_id, title, description, invoice_payload, provider_token, currency, prices,
+                     start_parameter, photo_url=None, photo_size=None, photo_width=None, photo_height=None,
+                     need_name=None, need_phone_number=None, need_email=None, need_shipping_address=None,
+                     is_flexible=None,
+                     disable_notification=None, reply_to_message_id=None, reply_markup=None):
+        result = apihelper.send_invoice(self.token, chat_id, title, description, invoice_payload, provider_token,
+                                        currency, prices, start_parameter, photo_url, photo_size, photo_width,
+                                        photo_height,
+                                        need_name, need_phone_number, need_email, need_shipping_address, is_flexible,
+                                        disable_notification, reply_to_message_id, reply_markup)
+        return types.Message.de_json(result)
+
+    def answer_shipping_query(self, shipping_query_id, ok, shipping_options=None, error_message=None):
+        return apihelper.answer_shippingQuery(self.token, shipping_query_id, ok, shipping_options, error_message)
+
+    def answer_pre_checkout_query(self, pre_checkout_query_id, ok, error_message=None):
+        return apihelper.answer_pre_checkout_query(self.token, pre_checkout_query_id, ok, error_message)
+
     def edit_message_caption(self, caption, chat_id=None, message_id=None, inline_message_id=None, reply_markup=None):
         result = apihelper.edit_message_caption(self.token, caption, chat_id, message_id, inline_message_id,
                                                 reply_markup)
@@ -839,6 +905,28 @@ class TeleBot:
     def add_callback_query_handler(self, handler_dict):
         self.callback_query_handlers.append(handler_dict)
 
+    def shipping_query_handler(self, func, **kwargs):
+        def decorator(handler):
+            handler_dict = self._build_handler_dict(handler, func=func, **kwargs)
+            self.add_shipping_query_handler(handler_dict)
+            return handler
+
+        return decorator
+
+    def add_shipping_query_handler(self, handler_dict):
+        self.shipping_query_handlers.append(handler_dict)
+
+    def pre_checkout_query_handler(self, func, **kwargs):
+        def decorator(handler):
+            handler_dict = self._build_handler_dict(handler, func=func, **kwargs)
+            self.add_pre_checkout_query_handler(handler_dict)
+            return handler
+
+        return decorator
+
+    def add_pre_checkout_query_handler(self, handler_dict):
+        self.pre_checkout_query_handlers.append(handler_dict)
+
     def _test_message_handler(self, message_handler, message):
         for filter, filter_value in six.iteritems(message_handler['filters']):
             if filter_value is None:
@@ -875,10 +963,38 @@ class AsyncTeleBot(TeleBot):
     def get_me(self):
         return TeleBot.get_me(self)
 
+    @util.async()
+    def get_file(self, *args):
+        return TeleBot.get_file(self, *args)
+
+    @util.async()
+    def download_file(self, *args):
+        return TeleBot.download_file(self, *args)
+
     @util.async()
     def get_user_profile_photos(self, *args, **kwargs):
         return TeleBot.get_user_profile_photos(self, *args, **kwargs)
 
+    @util.async()
+    def get_chat(self, *args):
+        return TeleBot.get_chat(self, *args)
+
+    @util.async()
+    def leave_chat(self, *args):
+        return TeleBot.leave_chat(self, *args)
+
+    @util.async()
+    def get_chat_administrators(self, *args):
+        return TeleBot.get_chat_administrators(self, *args)
+
+    @util.async()
+    def get_chat_members_count(self, *args):
+        return TeleBot.get_chat_members_count(self, *args)
+
+    @util.async()
+    def get_chat_member(self, *args):
+        return TeleBot.get_chat_member(self, *args)
+
     @util.async()
     def send_message(self, *args, **kwargs):
         return TeleBot.send_message(self, *args, **kwargs)
@@ -887,6 +1003,10 @@ class AsyncTeleBot(TeleBot):
     def forward_message(self, *args, **kwargs):
         return TeleBot.forward_message(self, *args, **kwargs)
 
+    @util.async()
+    def delete_message(self, *args):
+        return TeleBot.delete_message(self, *args)
+
     @util.async()
     def send_photo(self, *args, **kwargs):
         return TeleBot.send_photo(self, *args, **kwargs)
@@ -895,6 +1015,10 @@ class AsyncTeleBot(TeleBot):
     def send_audio(self, *args, **kwargs):
         return TeleBot.send_audio(self, *args, **kwargs)
 
+    @util.async()
+    def send_voice(self, *args, **kwargs):
+        return TeleBot.send_voice(self, *args, **kwargs)
+
     @util.async()
     def send_document(self, *args, **kwargs):
         return TeleBot.send_document(self, *args, **kwargs)
@@ -907,10 +1031,74 @@ class AsyncTeleBot(TeleBot):
     def send_video(self, *args, **kwargs):
         return TeleBot.send_video(self, *args, **kwargs)
 
+    @util.async()
+    def send_video_note(self, *args, **kwargs):
+        return TeleBot.send_video_note(self, *args, **kwargs)
+
     @util.async()
     def send_location(self, *args, **kwargs):
         return TeleBot.send_location(self, *args, **kwargs)
 
+    @util.async()
+    def send_venue(self, *args, **kwargs):
+        return TeleBot.send_venue(self, *args, **kwargs)
+
+    @util.async()
+    def send_contact(self, *args, **kwargs):
+        return TeleBot.send_contact(self, *args, **kwargs)
+
     @util.async()
     def send_chat_action(self, *args, **kwargs):
         return TeleBot.send_chat_action(self, *args, **kwargs)
+
+    @util.async()
+    def kick_chat_member(self, *args):
+        return TeleBot.kick_chat_member(self, *args)
+
+    @util.async()
+    def unban_chat_member(self, *args):
+        return TeleBot.unban_chat_member(self, *args)
+
+    @util.async()
+    def edit_message_text(self, *args, **kwargs):
+        return TeleBot.edit_message_text(self, *args, **kwargs)
+
+    @util.async()
+    def edit_message_reply_markup(self, *args, **kwargs):
+        return TeleBot.edit_message_reply_markup(self, *args, **kwargs)
+
+    @util.async()
+    def send_game(self, *args, **kwargs):
+        return TeleBot.send_game(self, *args, **kwargs)
+
+    @util.async()
+    def set_game_score(self, *args, **kwargs):
+        return TeleBot.set_game_score(self, *args, **kwargs)
+
+    @util.async()
+    def get_game_high_scores(self, *args, **kwargs):
+        return TeleBot.get_game_high_scores(self, *args, **kwargs)
+
+    @util.async()
+    def send_invoice(self, *args, **kwargs):
+        return TeleBot.send_invoice(self, *args, **kwargs)
+
+    @util.async()
+    def answer_shipping_query(self, *args, **kwargs):
+        return TeleBot.answer_shipping_query(self, *args, **kwargs)
+
+    @util.async()
+    def answer_pre_checkout_query(self, *args, **kwargs):
+        return TeleBot.answer_pre_checkout_query(self, *args, **kwargs)
+
+    @util.async()
+    def edit_message_caption(self, *args, **kwargs):
+        return TeleBot.edit_message_caption(self, *args, **kwargs)
+
+    @util.async()
+    def answer_inline_query(self, *args, **kwargs):
+        return TeleBot.answer_inline_query(self, *args, **kwargs)
+
+    @util.async()
+    def answer_callback_query(self, *args, **kwargs):
+        return TeleBot.answer_callback_query(self, *args, **kwargs)
